package testresume.utils;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.tcf.internal.debug.model.TCFLaunch;
import org.eclipse.tcf.protocol.IChannel;
import org.eclipse.tcf.protocol.IToken;
import org.eclipse.tcf.services.IRunControl;
import org.eclipse.tcf.services.IRunControl.RunControlContext;


import testresume.plugin.TestResumePlugin;

public class Utils {
	
	private static void log(int severity, int code, String message, Throwable exception) 
	{
		String pluginID = TestResumePlugin.getDefault().getBundle().getSymbolicName();
		IStatus status = new Status(
				severity,
				pluginID,
				code,
				message,
				exception);
		TestResumePlugin.getDefault().getLog().log(status);
	}	
	
	public static void log(Exception e) {
		log(IStatus.ERROR, IStatus.OK, "Unexpected Exception", e); //$NON-NLS-1$ 
	}
	
	/**
	 * 
	 * @param context the context to suspend\resume
	 * @param suspend suspend\resume
	 */
	private static void suspendResume(final RunControlContext context, final boolean suspend) {
		if (suspend && context.canSuspend()) {
			context.suspend(new IRunControl.DoneCommand() {					
				@Override
				public void doneCommand(IToken token, Exception error) {
					if (error != null) {
						log(error);
					}
				}
			});			
		} else if (!suspend && context.canResume(IRunControl.RM_RESUME)) {
			context.resume(IRunControl.RM_RESUME, 0, new IRunControl.DoneCommand() {					
				@Override
				public void doneCommand(IToken token, Exception error) {
					if (error != null) {
						log(error);	
					}					
				}
			});
		}		
	}

	/**
	 * 
	 * @param context_id the context id of the context to suspend\resume
	 * @param suspend suspend\resume
	 * @param runControl the service to use for locating the context related to context_id
	 */
	private static void suspendResume(final String context_id, final boolean suspend, final IRunControl runControl) {
		runControl.getContext(context_id, new IRunControl.DoneGetContext() {				
			@Override
			public void doneGetContext(IToken token, Exception error, RunControlContext context) {
				if (error == null) {
					suspendResume(context, suspend);
				} else {
					log(error);
				}					
			}
		});
	}
	
	/**
	 * 
	 * @return the IRunControl remote service of the first found launch (assuming it is TCFLaunch and its channel is in IChannel.STATE_OPEN state. 
	 */
	private static IRunControl getRunControl() {
		IRunControl retval = null;
		ILaunchManager lm = DebugPlugin.getDefault().getLaunchManager();
		ILaunch[] launches = lm.getLaunches();
		if (launches.length > 0) {
			ILaunch launch = launches[0];						
			if ((launch != null) && (launch instanceof TCFLaunch)) { 
				TCFLaunch tcfLaunch = (TCFLaunch)launch;
				IChannel channel = tcfLaunch.getChannel();
				if ((channel != null) && (channel.getState() == IChannel.STATE_OPEN)) {
					retval = channel.getRemoteService(IRunControl.class);
				}
			}
		}				
		return retval;
	}
				

	/**
	 * Suspend\Resume the first found launch, in case it is TCFLaunch and its channel is opened and supplies IRunControl service.
	 * Track all the root contexts of the launch and suspend\resume them. 
	 * @param suspend if true - suspend the launch. Otherwise - resume it.
	 */
	public static void suspendResume(final boolean suspend) {
		final IRunControl runControl = getRunControl();
		if (runControl != null) {
			//Find root contexts
			runControl.getChildren(null, new IRunControl.DoneGetChildren() {									
				@Override
				public void doneGetChildren(IToken token, Exception error, String[] context_ids) {
					if (error == null) {
						for (String context_id : context_ids) {
							suspendResume(context_id, suspend, runControl);
						}
					} else { //Report error
						log(error);
					}
				}
			});
		}
	}
}
